// src/host/renderer.tsx
import { Application, Pool } from "pr1";

// ../shared/lib/error.js
function createErrorWithCode(message, code) {
  let err = new Error(message);
  err.code = code;
  return err;
}

// ../shared/lib/defer.js
function defer() {
  let resolve;
  let reject;
  let promise = new Promise((_resolve, _reject) => {
    resolve = _resolve;
    reject = _reject;
  });
  return { promise, resolve, reject };
}

// ../shared/lib/client.js
var Client = class {
  backend;
  channels = /* @__PURE__ */ new Map();
  closedDeferred = defer();
  messageCallback = null;
  nextRequestId = 65536;
  requests = /* @__PURE__ */ new Map();
  userClose;
  userClosing = false;
  info = null;
  initializationData = null;
  state = null;
  identifier = null;
  staticUrl = null;
  version = null;
  constructor(backend, options) {
    this.backend = backend;
    this.userClose = options?.close ?? null;
  }
  close() {
    if (this.userClose) {
      this.closedDeferred.resolve(this.userClose());
      this.userClosing = true;
    } else {
      this.backend.close();
    }
  }
  get closed() {
    return (async () => {
      await Promise.race([
        this.backend.closed,
        this.closedDeferred.promise
      ]);
      if (this.userClosing) {
        await this.closedDeferred.promise;
      }
    })();
  }
  listen(channelId) {
    if (this.channels.has(channelId)) {
      throw new Error(`Already listening on channel ${channelId}`);
    }
    let channelData = {
      deferred: null,
      queue: []
    };
    this.channels.set(channelId, channelData);
    return {
      async *[Symbol.asyncIterator]() {
        while (true) {
          if (channelData.queue.length < 1) {
            if (!channelData.deferred) {
              channelData.deferred = defer();
            }
            await channelData.deferred.promise;
          }
          yield channelData.queue.pop();
        }
      },
      close: () => {
        this.channels.delete(channelId);
      },
      send: (data) => {
        this.backend.send({
          type: "channel",
          id: channelId,
          data
        });
      }
    };
  }
  async initialize() {
    let initializationMessage;
    let stateMessage;
    try {
      let createProtocolError = () => createErrorWithCode("Invalid message", "APP_PROTOCOL");
      let messages = this.backend.messages;
      let item1 = await messages.next();
      if (item1.done || item1.value.type !== "initialize") {
        throw createProtocolError();
      }
      initializationMessage = item1.value;
      let item2 = await messages.next();
      if (item2.done || item2.value.type !== "state") {
        throw createProtocolError();
      }
      stateMessage = item2.value;
    } catch (err) {
      switch (err.code) {
        case "APP_PROTOCOL":
          return {
            ok: false,
            reason: "invalid_protocol"
          };
        default:
          throw err;
      }
    }
    this.info = {
      clientId: initializationMessage.clientId,
      identifier: initializationMessage.identifier,
      staticUrl: initializationMessage.staticUrl,
      version: initializationMessage.version
    };
    this.initializationData = initializationMessage;
    this.state = stateMessage.data;
    return {
      ok: true,
      identifier: initializationMessage.identifier,
      name: stateMessage.data.info.name,
      staticUrl: initializationMessage.staticUrl,
      version: initializationMessage.version
    };
  }
  onMessage(callback) {
    this.messageCallback = callback;
  }
  request = async (data) => {
    let requestId = this.nextRequestId++;
    let deferred = defer();
    this.requests.set(requestId, deferred);
    this.backend.send({
      type: "request",
      id: requestId,
      data
    });
    return await deferred.promise;
  };
  sendRawMessage(message) {
    this.backend.send(message);
  }
  async start() {
    try {
      for await (let message of { [Symbol.asyncIterator]: () => this.backend.messages }) {
        switch (message.type) {
          case "channel": {
            let channelData = this.channels.get(message.id);
            if (channelData) {
              channelData.queue.push(message.data);
              if (channelData.deferred) {
                channelData.deferred.resolve();
                channelData.deferred = null;
              }
            }
            break;
          }
          case "response": {
            let request = this.requests.get(message.id);
            if (request) {
              request.resolve(message.data);
              this.requests.delete(message.id);
            }
            break;
          }
          case "state": {
            this.state = {
              ...this.state,
              ...message.data
            };
            break;
          }
        }
        this.messageCallback?.(message);
      }
    } finally {
      for (let request of this.requests.values()) {
        request.reject(createErrorWithCode("Closed", "APP_CLOSED"));
      }
      this.requests.clear();
    }
  }
};

// src/host/renderer.tsx
import { createRoot } from "react-dom/client";
import { Component } from "react";

// src/shared/context-menu.tsx
import { ContextMenuContext, React } from "pr1";
import { jsx } from "react/jsx-runtime";
var NativeContextMenu = class extends React.Component {
  constructor(props) {
    super(props);
    this.props.triggerRef.current = (event) => {
      let menu = this.props.createMenu(event);
      let position = {
        x: event.clientX,
        y: event.clientY
      };
      window.api.main.triggerContextMenu(menu, position).then((selectedPath) => {
        if (selectedPath !== null) {
          this.props.onSelect(selectedPath);
        }
        this.props.onClose(selectedPath !== null);
      });
    };
  }
  render() {
    return null;
  }
};
function NativeContextMenuProvider(props) {
  return /* @__PURE__ */ jsx(ContextMenuContext.Provider, { value: NativeContextMenu, children: props.children });
}

// src/host/app-backend.ts
import { MemoryStore, SnapshotProvider, deserialize, serialize } from "pr1";

// src/host/navigation.ts
var oldNavigation = window.navigation;
var newNavigation = {
  currentEntry: {
    getState: () => void 0,
    url: "file:///"
  },
  addEventListener(type, listener) {
    oldNavigation.addEventListener("navigate", (event) => {
      event.preventDefault();
      let defaultPrevented = false;
      listener({
        destination: event.destination,
        canIntercept: true,
        info: event.info?.info,
        intercept({ handler }) {
          handler();
        },
        preventDefault: () => {
          defaultPrevented = true;
        }
      });
      if (!defaultPrevented) {
        newNavigation.currentEntry = {
          getState: () => event.info?.state,
          url: event.destination.url
        };
      }
    });
  },
  navigate(url, options) {
    oldNavigation.navigate(url, {
      info: {
        info: options?.info,
        state: options?.state
      }
    }).committed.catch(() => {
    });
  }
};
window.navigation = newNavigation;

// src/host/app-backend.ts
var ElectronAppBackend = class extends SnapshotProvider {
  _drafts = {};
  _createSnapshot() {
    return {
      drafts: Object.fromEntries(
        Object.values(this._drafts).map((draftInstance) => [draftInstance.id, draftInstance.getSnapshot()])
      )
    };
  }
  async initialize() {
    let draftSkeletons = await window.api.drafts.list();
    this._drafts = Object.fromEntries(
      draftSkeletons.map((skeleton) => {
        let instance = new ElectronAppDraftInstance(this, skeleton);
        return [instance.id, instance];
      })
    );
    this._update();
  }
  createStore(name, options) {
    switch (options.type) {
      case "persistent":
        return new ElectronAppStore(name);
      case "session":
        return new MemoryStore();
    }
  }
  async createDraft(contents) {
    let skeleton = await window.api.drafts.create(contents);
    if (skeleton) {
      let instance = new ElectronAppDraftInstance(this, skeleton);
      this._drafts[instance.id] = instance;
      this._update();
      return instance;
    }
    return null;
  }
  async queryDraft() {
    let skeleton = await window.api.drafts.query();
    if (skeleton) {
      let instance = new ElectronAppDraftInstance(this, skeleton);
      this._drafts[instance.id] = instance;
      this._update();
      return instance;
    }
    return null;
  }
};
var ElectronAppDraftInstance = class extends SnapshotProvider {
  constructor(_appBackend, skeleton) {
    super();
    this._appBackend = _appBackend;
    this.id = skeleton.id;
    this._entryPath = skeleton.entryPath;
    this._name = skeleton.name;
  }
  id;
  _entryPath;
  _name;
  get _draftEntryId() {
    return this.id;
  }
  _createSnapshot() {
    return {
      model: this,
      id: this.id,
      name: this._name
    };
  }
  getEntryDocumentSlot() {
    return new ElectronAppDocumentSlot(this._entryPath);
  }
  async remove() {
    await window.api.drafts.delete(this._draftEntryId);
    delete this._appBackend._drafts[this.id];
    this._appBackend._update();
  }
  async setName(name) {
    this._name = name;
    this._update();
    this._appBackend._update();
    await window.api.drafts.setName(this._draftEntryId, name);
  }
};
var ElectronAppDocumentSlot = class extends SnapshotProvider {
  constructor(_path) {
    super();
    this._path = _path;
    this.id = this._path;
  }
  id;
  _instance = null;
  _status = "unwatched";
  _createSnapshot() {
    return {
      model: this,
      id: this.id,
      instance: this._instance && {
        ...this._instance
      },
      path: this._path.split(window.api.platform === "win32" ? "\\" : "/"),
      status: this._status
    };
  }
  async open() {
    await window.api.drafts.openFile(this._path);
  }
  async reveal() {
    await window.api.drafts.revealFile(this._path);
  }
  async watch(options) {
    this._status = "loading";
    window.api.drafts.watch(this._path, (change) => {
      this._status = change.status;
      this._instance = change.instance && {
        ...this._instance,
        contents: change.instance.contents,
        lastExternalModificationDate: change.instance.lastExternalModificationDate,
        lastModificationDate: change.instance.lastModificationDate,
        possiblyWritable: true,
        writable: true
      };
      this._update();
    }, (callback) => {
      options.signal.addEventListener("abort", () => void callback());
    });
  }
  async write(contents) {
    await window.api.drafts.write(this._path, contents);
  }
};
var ElectronAppStore = class {
  constructor(_name) {
    this._name = _name;
  }
  async read(key) {
    let value = await window.api.store.read(this._name, key);
    return value !== void 0 ? deserialize(value) : void 0;
  }
  async *readAll() {
    for (let [key, value] of await window.api.store.readAll(this._name)) {
      yield [key, deserialize(value)];
    }
  }
  async write(key, value) {
    await window.api.store.write(this._name, key, serialize(value));
  }
};

// src/host/renderer.tsx
import { jsx as jsx2 } from "react/jsx-runtime";
var App = class extends Component {
  appBackend = new ElectronAppBackend();
  client = null;
  hostSettingsId = new URL(location.href).searchParams.get("hostSettingsId");
  pool = new Pool();
  constructor(props) {
    super(props);
    this.state = {
      hostSettings: null
    };
  }
  componentDidMount() {
    this.pool.add(async () => {
      let { hostSettingsRecord } = await window.api.hostSettings.list();
      let hostSettings = hostSettingsRecord[this.hostSettingsId];
      this.client = await createLocalClient(hostSettings);
      this.setState({ hostSettings });
    });
  }
  render() {
    if (!this.state.hostSettings) {
      return /* @__PURE__ */ jsx2("div", {});
    }
    return /* @__PURE__ */ jsx2(NativeContextMenuProvider, { children: /* @__PURE__ */ jsx2(
      Application,
      {
        appBackend: this.appBackend,
        client: this.client,
        hostInfo: {
          id: this.state.hostSettings.id,
          imageUrl: null,
          description: null,
          label: this.state.hostSettings.label,
          local: true
        },
        onHostStarted: () => {
          this.pool.add(async () => {
            await document.fonts.load("12px Material Symbols Rounded");
            await document.fonts.load("12px Material Symbols Sharp");
            window.api.main.ready();
          });
        }
      }
    ) });
  }
};
document.body.dataset["platform"] = window.api.platform;
var root = createRoot(document.getElementById("root"));
root.render(/* @__PURE__ */ jsx2(App, {}));
async function createLocalClient(hostSettings) {
  let messageDeferred = null;
  let messageQueue = [];
  let client = new Client({
    close: () => {
    },
    closed: new Promise(() => {
    }),
    messages: async function* () {
      while (true) {
        if (messageQueue.length < 1) {
          messageDeferred = defer();
          await messageDeferred.promise;
        }
        yield messageQueue.shift();
      }
    }(),
    send: (message) => void window.api.host.sendMessage(hostSettings.id, message)
  });
  window.api.host.onMessage((message) => {
    messageQueue.push(message);
    if (messageDeferred) {
      messageDeferred.resolve();
      messageDeferred = null;
    }
  });
  let initialization = client.initialize();
  await window.api.host.ready(hostSettings.id);
  let result = await initialization;
  if (!result.ok) {
    throw new Error("Failed to connect to local client");
  }
  return client;
}
export {
  App
};
